@@ -82,6 +82,10 @@ onak_status_t parse_subpackets(unsigned char *data, size_t len,
 			packetlen <<= 8;
 			packetlen |= data[offset++];
 		}
+		/* Check the supplied length is within the remaining data */
+		if (packetlen == 0 || (packetlen + offset) > length) {
+			return ONAK_E_INVALID_PKT;
+		}
 		switch (data[offset] & 0x7F) {
 		case OPENPGP_SIGSUB_CREATION:
 			/*
