@@ -207,6 +207,11 @@ struct onak_dbctx {
 			void (*iterfunc)(void *ctx,
 			struct openpgp_publickey *key),	void *ctx);
 
+/**
+ * @brief Configuration file information for this backend instance
+ */
+	struct onak_db_config *config;
+
 /**
  * @brief Private backend context information.
  */
