@@ -206,13 +206,21 @@ static void dynamic_cleanupdb(struct onak_dbctx *dbctx)
 	}
 }
 
-struct onak_dbctx *keydb_dynamic_init(bool readonly)
+struct onak_dbctx *keydb_dynamic_init(struct onak_db_config *dbcfg,
+		bool readonly)
 {
 	struct onak_dbctx *dbctx;
 	char *soname;
 	char *initname;
-	struct onak_dbctx *(*backend_init)(bool);
+	struct onak_dbctx *(*backend_init)(struct onak_db_config *, bool);
 	struct onak_dynamic_dbctx *privctx;
+	char *type;
+
+	if (dbcfg == NULL) {
+		logthing(LOGTHING_CRITICAL,
+			"No backend database configuration supplied.");
+		return NULL;
+	}
 
 	dbctx = malloc(sizeof(struct onak_dbctx));
 
@@ -220,15 +228,16 @@ struct onak_dbctx *keydb_dynamic_init(bool readonly)
 		return NULL;
 	}
 
+	dbctx->config = dbcfg;
 	dbctx->priv = privctx = malloc(sizeof(struct onak_dynamic_dbctx));
 	if (dbctx->priv == NULL) {
 		free(dbctx);
 		return (NULL);
 	}
 
+	type = dbcfg->type;
 	if (config.use_keyd) {
-		free(config.db_backend);
-		config.db_backend = strdup("keyd");
+		type = "keyd";
 	}
 
 	if (!config.db_backend) {
@@ -237,21 +246,21 @@ struct onak_dbctx *keydb_dynamic_init(bool readonly)
 	}
 
 	if (config.backends_dir == NULL) {
-		soname = malloc(strlen(config.db_backend)
+		soname = malloc(strlen(type)
 			+ strlen("./libkeydb_")
 			+ strlen(".so")
 			+ 1);
 
-		sprintf(soname, "./libkeydb_%s.so", config.db_backend);
+		sprintf(soname, "./libkeydb_%s.so", type);
 	} else {
-		soname = malloc(strlen(config.db_backend)
+		soname = malloc(strlen(type)
 			+ strlen("/libkeydb_")
 			+ strlen(".so")
 			+ strlen(config.backends_dir)
 			+ 1);
 
 		sprintf(soname, "%s/libkeydb_%s.so", config.backends_dir,
-			config.db_backend);
+			type);
 	}
 
 	logthing(LOGTHING_INFO, "Loading dynamic backend: %s", soname);
@@ -270,7 +279,7 @@ struct onak_dbctx *keydb_dynamic_init(bool readonly)
 			+ strlen("keydb_")
 			+ strlen("_init")
 			+ 1);
-	sprintf(initname, "keydb_%s_init", config.db_backend);
+	sprintf(initname, "keydb_%s_init", type);
 
 	*(void **) (&backend_init) = dlsym(privctx->backend_handle, initname);
 	free(initname);
@@ -286,7 +295,7 @@ struct onak_dbctx *keydb_dynamic_init(bool readonly)
 	free(soname);
 	soname = NULL;
 
-	privctx->loadeddbctx = backend_init(readonly);
+	privctx->loadeddbctx = backend_init(dbcfg, readonly);
 
 	if (privctx->loadeddbctx != NULL) {
 		dbctx->cleanupdb = dynamic_cleanupdb;
