@@ -102,7 +102,7 @@ int main(int argc, char *argv[])
 	readconfig(configfile);
 	free(configfile);
 	initlogthing("maxpath", config.logfile);
-	dbctx = config.dbinit(true);
+	dbctx = config.dbinit(config.backend, true);
 	if (dbctx != NULL) {
 		inithash();
 		findmaxpath(dbctx, 30);
