@@ -308,7 +308,7 @@ onak_status_t read_openpgp_stream(int (*getchar_func)(void *ctx, size_t count,
 						break;
 					}
 					curpacket->packet->length = 
-						(curchar << 24);
+						((unsigned) curchar << 24);
 					if (getchar_func(ctx, 1, &curchar)) {
 						rc = ONAK_E_INVALID_PKT;
 						break;
@@ -354,12 +354,62 @@ onak_status_t read_openpgp_stream(int (*getchar_func)(void *ctx, size_t count,
 		} else {
 			rc = ONAK_E_INVALID_PKT;
 		}
+		if (rc == ONAK_E_OK) {
+			/* Make sure the packet version is sane */
+			switch (curpacket->packet->tag) {
+			case OPENPGP_PACKET_ENCRYPTED_MDC:
+				/* These packets must be v1 */
+				if (curpacket->packet->data[0] != 1) {
+					rc = ONAK_E_INVALID_PKT;
+				}
+				break;
+			case OPENPGP_PACKET_PKSESSIONKEY:
+			case OPENPGP_PACKET_ONEPASSSIG:
+				/* These packets must be v3 */
+				if (curpacket->packet->data[0] != 3) {
+					rc = ONAK_E_INVALID_PKT;
+				}
+				break;
+			case OPENPGP_PACKET_SYMSESSIONKEY:
+				/* These packets must be v4 */
+				if (curpacket->packet->data[0] != 4) {
+					rc = ONAK_E_INVALID_PKT;
+				}
+				break;
+			case OPENPGP_PACKET_SIGNATURE:
+			case OPENPGP_PACKET_SECRETKEY:
+			case OPENPGP_PACKET_PUBLICKEY:
+				/* Must be v2 -> v4 */
+				if (curpacket->packet->data[0] < 2 ||
+					curpacket->packet->data[0] > 4) {
+					rc = ONAK_E_INVALID_PKT;
+				}
+				break;
+			default:
+				break;
+			}
+		}
 	}
 
-	/* Trim the last packet if it doesn't actually exist */
-	if (packetend != NULL && (*packetend)->packet == NULL) {
-		free(*packetend);
-		*packetend = NULL;
+	if (packetend != NULL) {
+		if ((*packetend)->packet != NULL) {
+			/* If we got an invalid final packet, discard it. */
+			if ((*packetend)->packet->data != NULL &&
+					rc != ONAK_E_OK) {
+				free((*packetend)->packet->data);
+				(*packetend)->packet->data = NULL;
+			}
+			/* If we didn't get any data, clean it up. */
+			if ((*packetend)->packet->data == NULL) {
+				free((*packetend)->packet);
+				(*packetend)->packet = NULL;
+			}
+		}
+		/* Trim the last packet if it doesn't actually exist */
+		if ((*packetend)->packet == NULL) {
+			free(*packetend);
+			*packetend = NULL;
+		}
 	}
 
 	return (rc);
